/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Carlos Eduardo Wagner,
 *  Natan Martins Jory
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */

package gui;

import java.util.ArrayList;

public class Logica {

    private Pecas[][] tabuleiro;
    private boolean jogadorDaVez;
    private ArrayList<String> caminhos;
    private ArrayList<String> temCaminho;
    private ArrayList<String> caminhoTmp;
    private boolean diagonal[] = new boolean[4];

    public Logica(Pecas[][] tab) {
        this.tabuleiro = tab;
        this.jogadorDaVez = true;
        this.caminhos = new ArrayList<String>();
        this.temCaminho = new ArrayList<String>();
        this.caminhoTmp = new ArrayList<String>();
        this.diagonal[0] = false;
        this.diagonal[1] = false;
        this.diagonal[2] = false;
        this.diagonal[3] = false;

    }

    private boolean testaDiagonal(int dx, int dy) {
        /*
         * Executa o teste se a peça está endando na diagonal realizando a divisão por 2 e aproveitando o resto
         * O teste é executado a partir do resto da divisão da posição (X ou Y) por 2.
         */
        if ((((dy % 2 == 0) && (dx % 2 == 1)) || ((dy % 2 == 1) && (dx % 2 == 0)))) {
            return true;
        } else {
            return false;
        }
    }

    private boolean andaFrente(int ox, int oy, int dx, int dy) {
        /* Executa o teste para determinar se o jogador está andando para frente
        true amerelo, false vermelho

        Se o a diferença entre a origem e o destino, na coodernada x, for 1, o jogador está andando para frente
        O teste de y se dá para assegurar que o jogador está andando somente uma casa, porém nao iomporta a direção
         */
        if (this.jogadorDaVez == true) { //dy = oy +1
            if ((ox - dx == 1) && (oy - dy == 1 || oy - dy == -1)) {
                return true;
            } else {
                return false;
            }

            // Para o caso do jogador vermelho, a diferença entre origem e destino, na coordenada x, deve ser -1.
        } else {
            if ((ox - dx == - 1) && (oy - dy == 1 || oy - dy == -1)) {
                return true;
            } else {
                return false;
            }
        }

    }


    /**
     *
     * @param ox
     * @param oy
     * @param dx
     * @param dy
     * @return 0 - OK pode mover
     *         1 - Jogada Inválida
     */
    public int mover(int ox, int oy, int dx, int dy) {
        // dy - coluna, dx - linha


        if (tabuleiro[ox][oy].getRainha()) { // Movimento de rainha
            if (this.testaDiagonal(dx, dy)) { // faz o teste se está andando na diagonal
                if (this.comerMaiorCaminho(ox, oy, dx, dy) == 1) { // faz o teste para verificar se não há possibilidade de comer
                    return 0;
                }
                if (this.comerMaiorCaminho(ox, oy, dx, dy) == 3) {
                    this.jogadorDaVez = !this.jogadorDaVez;
                    return 0;
                } else {
                    return 1;
                }

            }

        } else { // Caso nao seja rainha, faz movimento de uma peça comum
            if (this.testaDiagonal(dx, dy)) { // teste para verificar se está andando na diagonal
                if (this.comerMaiorCaminho(ox, oy, dx, dy) == 1) { // faz o teste para verificar se não há possibilidade de comer
                    if (tabuleiro[ox][oy] != null) { // verifica que a origem é uma peça
                        if (tabuleiro[dx][dy] == null) { // verifica se o destino é nulo (espaço vago)
                            return 0;

                        } 
                    }
                }
            }
            if (this.comerMaiorCaminho(ox, oy, dx, dy) == 3) { // Caso tenha peça para comer:
                if (this.andaFrente(ox, oy, dx, dy)) { // verifica se está andando para frente
                    this.jogadorDaVez = !this.jogadorDaVez;
                    return 0;
                }
                return 1;
            }

        }
        return 1;

    }

    public boolean getJogadorDaVez() { // retorna true ou false, dependendo do jogador da vez
        return this.jogadorDaVez;
    }

    public ArrayList<String> procurarTodosCaminhos() {
        /*
         * Varre todo o tabuleiro procurando alguma peça que possa comer e que seja a peça do jogador da vez
         */
        String a = "";
        for (int i = 0; i < 8; i++) { // varre na posição x
            for (int j = 0; j < 8; j++) {// varre na posição y
                if (tabuleiro[i][j] != null) { // verifica se na posição das variáveis I e J, há uma peça
                    if (tabuleiro[i][j].getCor() == jogadorDaVez) { // verifica se esta peça faz parte do jogador da vez
                        if (this.verificaJogada(i, j)) { // verifica se a primeira peça que ela pode comer está a frente
                            this.percorreCaminho(i, j, i, j, i + "" + j, a); // verifica se tem mais algum caminho para esta peça, utilizando recursividade

                        }
                        if (tabuleiro[i][j].getRainha()) { // caso não, verifica se a peça é dama e tem caminho para comer
                            this.percorreCaminhoDama(i, j, i, j, i + "" + j, a,this.diagonal);

                        }

                    }
                }
            }
        }
        // arraylist com todos os caminhos encontrados
        ArrayList<String> copia = caminhos;

        // zerando caminhos
        caminhos = new ArrayList<String>();

        return copia;

    }

    /**
     * Método recursivo para verificar o maior caminho para caputra de peças
     *
     * @param antX linha de onde a peça veio
     * @param antY coluna de onde a peça veio
     * @param xAtual linha atual da peça
     * @param yAtual coluna atual da peça
     * @param anterior todo o caminho por onde a peça passou
     */
    public void percorreCaminho(int antX, int antY, int xAtual, int yAtual, String anterior, String tmp) {

        /* verificando direita acima */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual + 2 <= 7)) && ((xAtual - 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual + 1] != null) && (tabuleiro[xAtual - 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual + 1].getCor() != jogadorDaVez)) {
                    String tmp1;
                    tmp1 = tmp + "" + (xAtual - 1) + "" + (yAtual + 1);
                    this.caminhoTmp.add(tmp1);
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist

                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual + 2, novo, tmp1);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }

        /* verificando direita abaixo */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio

        if (((xAtual + 2 <= 7) && (yAtual + 2 <= 7)) && ((xAtual + 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual + 1] != null) && (tabuleiro[xAtual + 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual + 1].getCor() != jogadorDaVez)) {
                    String tmp1;
                    tmp1 = tmp + "" + (xAtual + 1) + "" + (yAtual + 1);
                    this.caminhoTmp.add(tmp1);
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual + 2, novo, tmp1);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }


        // esquerda acima
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual - 2 >= 0)) && ((xAtual - 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual - 1] != null) && (tabuleiro[xAtual - 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual - 1].getCor() != jogadorDaVez)) {
                    String tmp1;
                    tmp1 = tmp + "" + (xAtual - 1) + "" + (yAtual - 1);
                    this.caminhoTmp.add(tmp1);
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual - 2, novo, tmp1);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }

                }
            }
        }



        // esquerda abaixo
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual - 2 >= 0)) && ((xAtual + 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual - 1] != null) && (tabuleiro[xAtual + 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual - 1].getCor() != jogadorDaVez)) {
                    String tmp1;
                    tmp1 = tmp + "" + (xAtual + 1) + "" + (yAtual - 1);
                    this.caminhoTmp.add(tmp1);
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual - 2, novo, tmp1);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }
    }

    public boolean verificaJogada(int ox, int oy) {
        /*
         * Verifica se a primeira peça que pode ser comida está a frente da peça origem
         * Já cuidando para que não ultrapasse as margens do tabuleiro
         */
        int direcao = 0;

        if (jogadorDaVez == true) {
            direcao = -1;

        } else {
            direcao = 1;

        }
        // verifica 2 frente
        //ESQUERDA
        if (((oy >= 2) && (ox >= 2) && (direcao == -1)) || ((oy >= 2) && (ox <= 5) && (direcao == 1))) {
            if ((tabuleiro[ox + direcao][oy - 1] != null) && (tabuleiro[ox + direcao][oy - 1].getCor() != jogadorDaVez)) {
                if (tabuleiro[ox + 2 * direcao][oy - 2] == null) {
                    return true;
                }
            }
        }


        //DIREITA

        if (((oy <= 5) && (ox >= 2) && (direcao == -1)) || ((oy <= 5) && (ox <= 5) && (direcao == 1))) {
            if ((tabuleiro[ox + direcao][oy + 1] != null) && (tabuleiro[ox + direcao][oy + 1].getCor() != jogadorDaVez)) {
                if (tabuleiro[ox + 2 * direcao][oy + 2] == null) {
                    return true;
                }
            }
        }

        return false;

    }

    private int comerMaiorCaminho(int ox, int oy, int dx, int dy) {
        /*
         * Implementa a lógica para que se coma o maior caminho possível
         */
        ArrayList<String> todos = new ArrayList<String>();
        ArrayList<String> maiores = new ArrayList<String>();
        ArrayList<String> maioresPeca = new ArrayList<String>();

        todos = procurarTodosCaminhos(); // pega o Arraylist resultante que possui todos os caminhos
        maiores = this.maiorCaminho(todos); // seleciona somente os maiores
        maioresPeca = this.maiorCaminho(caminhoTmp); // seleciona somente o arraylist com o maior numero de peças
        this.temCaminho = maiores;
        System.out.println(maiores);
        for (String tmp : maiores) { // varre o array list de maiores caminhos

            int tamanhoString = tmp.length();
            String origem = tmp.substring(0, 2); // pega as duas primeiras posições da String
            String destino = tmp.substring(tamanhoString - 2, tamanhoString); // pega as duas ultiams posições da String
            String pecaSelecionada = ox + "" + oy;
            String destinoJogada = dx + "" + dy;

            if (pecaSelecionada.equals(origem) && destinoJogada.equals(destino)) { // caso o destino selecionado for igual o destino previsto no arraylist, poderá realizar a jogada
                for (int j = 0; j < maiores.size(); j++) { // faz uma varredura no ArrayList
                    if (tmp.equals(maiores.get(j))) {
                        String tmp2 = maioresPeca.get(j);
                        for (int k = 0; k < tmp2.length(); k += 2) { // elimina as peças comidas no caminho, apagando as peças contidas na String tmp2
                            int p = Integer.parseInt(tmp2.substring(k, k + 1));
                            int q = Integer.parseInt(tmp2.substring(k + 1, k + 2));
                            this.tabuleiro[p][q] = null;
                        }
                    }
                }

                caminhoTmp = new ArrayList<String>(); // zera o ArrayList caminhoTmp
                this.jogadorDaVez = !this.jogadorDaVez; // muda o jogador da vez
                return 1; // pode comer, jogada certa

            }
        }

        if (todos.size() > 0) {
            return 2; // tem um caminho que tem mais peças para comer
        } else {
            return 3; // não possui caminhos para comer
        }
    }

    public ArrayList<String> maiorCaminho(ArrayList<String> ca) {
        /*
         * Seleciona o maior caminho do Array list
         */
        int maior = -1;
        ArrayList<String> maiores = new ArrayList<String>();

        for (String linha : ca) { // varre o array list
            if (linha.length() > maior) { // caso o tamanho da String selecionada for maior que a variável maior
                maiores = new ArrayList<String>(); // limpa o arraylist
                maiores.add(linha); // adiciona a string no arraylist agora limpo
                maior = linha.length(); // muda o valor da variável maior, para o valor correspondente ao tamnho da String
            } else if (linha.length() == maior) { // caso o tamanho da String for igual ao da variável maior
                maiores.add(linha); // só adiciona mais um caminho
            }
        }
        return maiores;
    }
    // Trata a recursividade para Dama

    public void percorreCaminhoDama(int antX, int antY, int xAtual, int yAtual, String anterior, String peca, boolean diag[]) {
        // classe para a recursividade da Dama, faz o teste de toda a diagonal em que a dama se encontra e cuida para nao testar a mesma diagonal duas vezes (loop)
        this.diagonal[0] = diag[0]; // associa o vetor booleano qeu recebe com o que já existe na classe
        this.diagonal[0] = diag[1]; //
        this.diagonal[0] = diag[2]; //
        this.diagonal[0] = diag[3]; //

        for (int w = 8; w > 0; w--) {
            /* verificando direita acima */
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
            if (this.diagonal[0]==false){
            if (((xAtual - (w + 1) >= 0) && (yAtual + (w + 1) <= 7)) && ((xAtual - (w + 1) != antX) || (yAtual + (w + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual - w][yAtual + w] != null) && (tabuleiro[xAtual - (w + 1)][yAtual + (w + 1)] == null)) {
                    if ((tabuleiro[xAtual - w][yAtual + w].getCor() != jogadorDaVez)) {
                        // verificando se esta jogada não está presente no caminho percorrido até aqui
                        // proteção contra LOOP

                        if (!anterior.contains(antX + "" + antY + ":" + xAtual + "" + yAtual)) {
                            String pecas = peca + "" + (xAtual - w) + "" + (yAtual + w);

                            caminhoTmp.add(pecas);
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - (w + 1)) + "" + (yAtual + (w + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual - (w + 1)) + "" + (yAtual + (w + 1));
                                caminhos.add(novo);
                                this.diagonal[3]=true;
                                // será que deste ponto é possível capturar outra peça? vamos ver
                                this.percorreCaminhoDama(xAtual, yAtual, xAtual - (w + 1), yAtual + (w + 1), novo, pecas,this.diagonal);
                            } else {
                                // mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - (w + 1)) + "" + (yAtual + (w + 1)));
                                System.err.println("caminho: " + anterior);
                            }

                        }
                    }
                }
                }
            }

            if(this.diagonal[1]==false){
            /* verificando direita abaixo */
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio

            if (((xAtual + (w + 1) <= 7) && (yAtual + (w + 1) <= 7)) && ((xAtual + (w + 1) != antX) || (yAtual + (w + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual + w][yAtual + w] != null) && (tabuleiro[xAtual + (w + 1)][yAtual + (w + 1)] == null)) {
                    if ((tabuleiro[xAtual + w][yAtual + w].getCor() != jogadorDaVez)) {
                        // verificando se esta jogada não está presente no caminho percorrido até aqui
                        // proteção contra LOOP
                        if (!anterior.contains(antX + "" + antY + ":" + xAtual + "" + yAtual)) {
                            String pecas = peca + "" + (xAtual + w) + "" + (yAtual + w);

                            caminhoTmp.add(pecas);
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + (w + 1)) + "" + (yAtual + (w + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual + (w + 1)) + "" + (yAtual + (w + 1));
                                caminhos.add(novo);
                                this.diagonal[2] = true;
                                // será que deste ponto é possível capturar outra peça? vamos ver
                                this.percorreCaminhoDama(xAtual, yAtual, xAtual + (w + 1), yAtual + (w + 1), novo, pecas,this.diagonal);
                            } else {
                                // mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + (w + 1)) + "" + (yAtual + (w + 1)));
                                System.err.println("caminho: " + anterior);
                            }

                        }
                    }
                }
                }
            }

            if(this.diagonal[2]==false){
            // esquerda acima
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio

            if (((xAtual - (w + 1) >= 0) && (yAtual - (w + 1) >= 0)) && ((xAtual - (w + 1) != antX) || (yAtual - (w + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual - w][yAtual - w] != null) && (tabuleiro[xAtual - (w + 1)][yAtual - (w + 1)] == null)) {
                    if ((tabuleiro[xAtual - w][yAtual - w].getCor() != jogadorDaVez)) {
                        // verificando se esta jogada não está presente no caminho percorrido até aqui
                        // proteção contra LOOP
                        if (!anterior.contains(antX + "" + antY + ":" + xAtual + "" + yAtual)) {
                            String pecas = peca + "" + (xAtual - w) + "" + (yAtual - w);

                            caminhoTmp.add(pecas);
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - (w + 1)) + "" + (yAtual - (w + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual - (w + 1)) + "" + (yAtual - (w + 1));
                                caminhos.add(novo);
                                // será que deste ponto é possível capturar outra peça? vamos ver
                                this.diagonal[1]=true;
                                this.percorreCaminhoDama(xAtual, yAtual, xAtual - (w + 1), yAtual - (w + 1), novo, pecas,this.diagonal);
                            } else {
                                //mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - (w + 1)) + "" + (yAtual - (w + 1)));
                                System.err.println("caminho: " + anterior);
                            }

                        }
                        }
                    }
                }
            }



            // esquerda abaixo
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
            if (this.diagonal[3]==false){
            if (((xAtual + (w + 1) <= 7) && (yAtual - (w + 1) >= 0)) && ((xAtual + (w + 1) != antX) || (yAtual - (w + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual + w][yAtual - w] != null) && (tabuleiro[xAtual + (w + 1)][yAtual - (w + 1)] == null)) {
                    if ((tabuleiro[xAtual + w][yAtual - w].getCor() != jogadorDaVez)) {
                        // verificando se esta jogada não está presente no caminho percorrido até aqui
                        // proteção contra LOOP
                        if (!anterior.contains(antX + "" + antY + ":" + xAtual + "" + yAtual)) {
                            String pecas = peca + "" + (xAtual + w) + "" + (yAtual - w);

                            caminhoTmp.add(pecas);
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + (w + 1)) + "" + (yAtual - (w + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual + (w + 1)) + "" + (yAtual - (w + 1));
                                caminhoTmp.add(pecas);
                                caminhos.add(novo);
                                this.diagonal[0]=true;
                                // será que deste ponto é possível capturar outra peça? vamos ver
                                this.percorreCaminhoDama(xAtual, yAtual, xAtual + (w + 1), yAtual - (w + 1), novo, pecas, this.diagonal);
                            } else {
                                //mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + (w + 1)) + "" + (yAtual - (w + 1)));
                                System.err.println("caminho: " + anterior);
                            }

                        }
                    }
                }
                }
            }
        }
        // resetando os valores da variavel de teste de diagonal contrária
        this.diagonal[0] = false;
        this.diagonal[1] = false;
        this.diagonal[2] = false;
        this.diagonal[3] = false;
    }
}
