/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Carlos Eduardo Wagner,
 *  Natan Martins Jory
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */

package gui;

import java.awt.Color;
import java.awt.Graphics;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

public class Tabuleiro extends JPanel {

    private ImageIcon branca;
    private ImageIcon preta;
    private ImageIcon rainhaBranca;
    private ImageIcon rainhaPreta;
    private boolean preenchido;
    private Pecas[][] tabuleiro;
    int xPeca;
    int yPeca;
    boolean sPeca;
    private Logica logicaJogo;
    int selecionadoX, selecionadoY;
    private boolean jogadorDaVez;

    public Tabuleiro() {
        preta = new ImageIcon(this.getClass().getResource("face-angry.PNG")); // seleciona imagem para peça vermelha
        branca = new ImageIcon(this.getClass().getResource("face-worried.png")); // seleciona imagem apra peça amarela
        // mudar imagem para rainha
        rainhaPreta = new ImageIcon(this.getClass().getResource("red-queen.jpg"));// seleciona imagem para rainha vermelha
        rainhaBranca = new ImageIcon(this.getClass().getResource("yel-queen.jpg"));// seleciona imagem para rainha amarela

        // alterar a cor de fundo do painel
        this.setBackground(Color.WHITE);
        this.preenchido = false;

        this.tabuleiro = new Pecas[8][8];

        for (int lin = 0; lin < 8; lin++) { //percorrendo linhas
            // determinando a cor da peca
            // false - preta, true - branca
            boolean cor = (lin > 4) ? true : false;
            for (int col = 0; col < 8; col++) { //percorrendo colunas
                if ((lin != 3 && lin != 4) && (((lin % 2 == 0) && (col % 2 == 1)) || ((lin % 2 == 1) && (col % 2 == 0)))) {
                    this.tabuleiro[lin][col] = new Pecas(cor, col, lin);
                }
            }
        }
        this.jogadorDaVez = true;
        this.logicaJogo = new Logica(tabuleiro);

    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
    }

    public void limpar() {
        // fazendo o painel ser redesenhado
        this.repaint();
    }

    public void desenharTabuleiro() { // desenha o tabuleiro
        Graphics g = this.getGraphics();

        for (int i = 0; i < 400; i += 50) {
            for (int j = 0; j < 400; j += 50) {
                if (this.preenchido == false) {
                    g.setColor(Color.white);
                    g.fillRect(i, j, 50, 50);
                } else {
                    g.setColor(Color.black);
                    g.fillRect(i, j, 50, 50);
                }
                // novo valor sera' igual ao antigo valor negado
                this.preenchido = !this.preenchido;
            }
            this.preenchido = !this.preenchido;
        }
       }


    public void preencherPecas() { // preenche as peças no tabuleiro, cuidando para que seja nas casas pretas
        Graphics g = this.getGraphics();

        for (int lin = 0; lin
                < 8; lin++) {
            for (int col = 0; col
                    < 8; col++) {
                if (this.tabuleiro[lin][col] != null) {
                    if (tabuleiro[lin][col].getCor() == false) {
                        if (tabuleiro[lin][col].getRainha()==true){
                            g.drawImage(rainhaPreta.getImage(), 10 + (50 * col), 10 + (50 * lin), this);

                        }else{
                         g.drawImage(preta.getImage(), 10 + (50 * col), 10 + (50 * lin), this);
                        }
                    } else {
                        if (tabuleiro[lin][col].getRainha()==true){
                           g.drawImage(rainhaBranca.getImage(), 10 + (50 * col), 10 + (50 * lin), this);
                        }else{
                        g.drawImage(branca.getImage(), 10 + (50 * col), 10 + (50 * lin), this);
                        }
                    }
                }
            }
        }
    }

    public void mover(int y, int x) { // realiza o movimento da peça graficamente, a lógicaa do movimento é implementada na classe Logica
        Graphics gr = this.getGraphics();

        if (this.sPeca == false) { // seleciona a peça
            if (this.tabuleiro[x][y] != null) {
                if (this.tabuleiro[x][y].getCor() == this.logicaJogo.getJogadorDaVez()) {
                    gr.setColor(Color.ORANGE);
                    gr.drawRoundRect((y * 50) + 5, (x * 50) + 5, 40, 40, 5, 5);
                    sPeca = true;
                    this.xPeca = x;
                    this.yPeca = y;
                }
            }
        } else { // caso já tenha selecionado a peça, o jogador deseja movê-la

            if (this.tabuleiro[x][y] != null) { // se onde for clicado tiver uma peça do mesmo jogador, é selecionado esta nova peça
                if (this.tabuleiro[x][y].getCor() == this.logicaJogo.getJogadorDaVez()) {
                    this.desenharTabuleiro();
                    this.preencherPecas();
                    gr.setColor(Color.ORANGE);
                    gr.drawRoundRect((y * 50) + 5, (x * 50) + 5, 40, 40, 5, 5);
                    sPeca = true;
                    this.xPeca = x;
                    this.yPeca = y;
                }
            } else { //caso seja uam casa vazia, faz rotina de mover a peça e verifica a lógica na classe Logica, verificando se tem outras possibilidades.
                if (this.logicaJogo.mover(xPeca, yPeca, x, y) == 0) {
                    this.tabuleiro[x][y] = this.tabuleiro[xPeca][yPeca];
                    this.tabuleiro[xPeca][yPeca] = null;
                    this.fazerRainha(x, y);
                    this.desenharTabuleiro();
                    this.preencherPecas();
                    this.sPeca = false;
                    
                }

            }
        }
    }
    // Método não utilizado para realçar onde o jogador deve clicar, no caso de que o jogador seja obrigado a utilizar um determinado caminho
    /*
    public void desenhaDestino(int x, int y){
        Graphics gr1 = this.getGraphics();
        gr1.setColor(Color.ORANGE);
        gr1.drawRoundRect((y * 50) + 5, (x * 50) + 5, 40, 40, 5, 5);
    }
     
     */
        public void fazerRainha(int dx, int dy) { // verifica se a peça pode se tornar uma rainha (Dama)
        if (!this.logicaJogo.getJogadorDaVez()) { // caso a peça for amarela, será rainha quando atingir o outro lado do tabuleiro (x = 0)
            if (dx == 0) {
                tabuleiro[dx][dy].setRainha(true);
            }
        } else {
            if (dx == 7) {// caso a peça for vermelha, será rainha quando atingir o outro lado do tabuleiro (x = 7)
                tabuleiro[dx][dy].setRainha(true);
            }
        }
    }
}
